<?php namespace EscapeWork\Manager\Medias;

use EscapeWork\Resize\Resize;
use EscapeWork\Resize\ResizeException;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use EscapeWork\Manager\Medias\Models\MediaModelSize;
use Msg;
use Exception;
use InvalidArgumentException;
use App;
use Str;

class Upload
{

    /**
     * Variaveis de controle
     */
    protected
        $picture,
        $directory,
        $name,
        $mime,
        $size;

    /**
     * Tipos de imagens aceitas
     * @var array
     */
    private $acceptedTypes = array(
        'image/jpg', 'image/jpeg', 'image/pjpg', 'image/pjpeg', # jpg
        'image/png',                                            # png
    );

    public function setPicture($picture)
    {
        if (! $picture instanceof UploadedFile) {
            throw new InvalidArgumentException('A imagem precisa ser uma instância de UploadedFile');
        }

        $this->picture = $picture;
        return $this;
    }

    public function getPicture()
    {
        return $this->picture;
    }

    public function setDirectory($directory)
    {
        $this->directory = $directory;
        return $this;
    }

    public function getDirectory()
    {
        return $this->directory;
    }

    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    public function getName()
    {
        return $this->name;
    }

    public function setMime($mime)
    {
        $this->mime = $mime;
        return $this;
    }

    public function getMime()
    {
        return $this->mime;
    }

    public function setSize($size)
    {
        $this->size = $size;
        return $this;
    }

    public function getSize()
    {
        return $this->size;
    }

    /**
     * Validando o tamanho e os tipos da imagem
     */
    public function validate()
    {
        # mime type
        if (! in_array($this->getMime(), $this->acceptedTypes)) {
            throw new Exception('Apenas são aceitas imagens do tipo JPG e PNG.');
        }

        # diretório existe
        if (! is_dir($this->getDirectory())) {
            if (mkdir($this->getDirectory(), 0773)) {
                chmod($this->getDirectory(), 0773);
            } else {
                throw new Exception('O diretório <strong>'.$this->getDirectory().'</strong> não existe. Por favor, crie o mesmo.');
            }
        }

        # permissão de escrita
        if (! is_writable($this->getDirectory())) {
            throw new Exception('O diretório <strong>'.$this->getDirectory().'</strong> precisa de permissão de escrita.');
        }

        return $this;
    }

    /**
     * Fazendo o upload da imagem
     */
    public function upload()
    {
        $pic = static::formatString($this->picture->getClientOriginalName()) . '.' . $this->picture->guessExtension();

        while (is_file($this->getDirectory() . $pic)) {
            $pic = '_' . $pic;
        }

        $this->setName($pic);
        $this->picture->move($this->getDirectory(), $this->getName());
    }

    /**
     * Fazendo o upload e criando todos os tamanhos da imagem a partir do model
     *
     * @param  string       $directory
     * @param  UploadedFile $picture
     * @param  string       $model
     * @return mixed
     */
    public static function create($directory, $picture, $sizes = array())
    {
        try {
            $upload = new Upload();
            $upload->setDirectory($directory)
                  ->setPicture($picture)
                  ->setSize($upload->picture->getClientSize())
                  ->setMime($upload->picture->getClientMimeType())
                  ->validate()
                  ->upload();

            Resize::make(
                $upload->getDirectory(),
                $upload->getName(),
                $sizes
            );
        } catch (ResizeException $e) {
            return null;
        } catch (InvalidArgumentException $e) {
            return null;
        } catch (Exception $e) {
            Msg::setError($e->getMessage());

            return null;
        }

        return $upload;
    }

    /**
     * Formatando uma string, removendo acentos e caracteres especiais
     *
     * @param  string $string
     * @return string
     */
    public static function formatString($string)
    {
        $string  = explode('.', $string);
        $total   = count($string);
        unset($string[$total - 1]);

        return Str::slug(implode('.', $string));
    }
}
